@@ -4,6 +4,7 @@ from collections import OrderedDict
 
 from ..config import BuilderConfig, env_var_enabled
 from ..constants import BuildEnvVars
+from ..utils import safe_walk
 
 
 class IncludedFile(object):
@@ -127,6 +128,7 @@ class BuilderInterface(object):
 
             # Make sure reserved fields are set
             build_data.setdefault('artifacts', [])
+            build_data.setdefault('force-include', {})
 
             # Pass all the configured build hooks for future unforeseen scenarios needing ultimate control
             build_data['configured_build_hooks'] = configured_build_hooks
@@ -153,16 +155,23 @@ class BuilderInterface(object):
 
             yield artifact
 
-    def recurse_project_files(self):
+    def recurse_included_files(self):
         """
         Returns a consistently generated series of file objects for every file that should be distributed. Each file
         object has three `str` attributes:
 
         - `path` - the absolute path
-        - `relative_path` - the path relative to the project root
+        - `relative_path` - the path relative to the project root; will be an empty string for external files
         - `distribution_path` - the path to be distributed as
         """
-        for root, dirs, files in os.walk(self.root):
+        for project_file in self.recurse_project_files():
+            yield project_file
+
+        for explicit_file in self.recurse_explicit_files(self.config.get_force_include()):
+            yield explicit_file
+
+    def recurse_project_files(self):
+        for root, dirs, files in safe_walk(self.root):
             relative_path = os.path.relpath(root, self.root)
 
             # First iteration
@@ -185,6 +194,30 @@ class BuilderInterface(object):
                         os.path.join(root, f), relative_file_path, self.config.get_distribution_path(relative_file_path)
                     )
 
+    def recurse_explicit_files(self, inclusion_map):
+        for source, target_path in inclusion_map.items():
+            external = not source.startswith(self.root)
+            if os.path.isfile(source):
+                yield IncludedFile(source, '' if external else os.path.relpath(source, self.root), target_path)
+            elif os.path.isdir(source):
+                for root, dirs, files in safe_walk(source):
+                    relative_path = os.path.relpath(root, source)
+
+                    # First iteration
+                    if relative_path == '.':
+                        relative_path = ''
+
+                    dirs.sort()
+                    files.sort()
+
+                    for f in files:
+                        relative_file_path = os.path.join(relative_path, f)
+                        yield IncludedFile(
+                            os.path.join(root, f),
+                            '' if external else os.path.relpath(relative_file_path, self.root),
+                            os.path.join(target_path, relative_file_path),
+                        )
+
     @property
     def root(self):
         """
@@ -315,7 +348,9 @@ class BuilderInterface(object):
         for hook_name, config in self.config.hook_config.items():
             build_hook = self.plugin_manager.build_hook.get(hook_name)
             if build_hook is None:
-                raise ValueError('Unknown build hook: {}'.format(hook_name))
+                from ...plugin.exceptions import UnknownPluginError
+
+                raise UnknownPluginError('Unknown build hook: {}'.format(hook_name))
 
             configured_build_hooks[hook_name] = build_hook(
                 self.root, config, self.config, self.metadata, directory, self.PLUGIN_NAME, self.app
@@ -366,6 +401,6 @@ class BuilderInterface(object):
     @staticmethod
     def normalize_file_name_component(file_name):
         """
-        https://www.python.org/dev/peps/pep-0427/#escaping-and-unicode
+        https://peps.python.org/pep-0427/#escaping-and-unicode
         """
         return re.sub(r'[^\w\d.]+', '_', file_name, re.UNICODE)
